package	jp.co.sds_corp.fixconnect.message;

import java.io.ByteArrayOutputStream;
import java.util.Iterator;
import java.util.Vector;

import jp.co.sds_corp.fixconnect.FIXException;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;

public class FIXParser
{
	DictionaryLoader dict;
	Vector<Field>	vecDef = new Vector<Field>();

	protected FIXParser() {
	}

	public void init( IConfiguration config ) throws Exception {
		IConfiguration attr = config.getAttributes();
		String var;
		var = attr.get( "debug", "false" );
		if ( var.equals( "true" )) {
			setDebug( true );
		}
		IConfiguration c;
		c = config.getChild( "Dictionary" );
		Class cls = Class.forName( c.getAttributes().get( "class" ) );
		dict = (DictionaryLoader )cls.newInstance();
		dict.init( c );

		c = config.getChild( "Default" );
		if ( c != null ) {
			readDefault( c );
		}
	}

	void readDefault( IConfiguration config ) throws Exception {
		Iterator<IConfiguration> itr = config.getChilds( "tag" );
		while ( itr.hasNext()) {
			IConfiguration ct = itr.next();
			IConfiguration defs = ct.getAttributes();
			Tag	tag = null;
			String name = null;
			String id = defs.get( "id", null );
			if ( id != null ) {
				tag = dict.findTag( id );
			} else {
				name = defs.get( "name", null );
				if ( name != null ) {
					tag = dict.findTagName( name );
				}
			}
			if ( tag == null ) {
				throw( new Exception( "default tag[" + id + "," + name
						+ "] not found." ));
			}
			String value = defs.get( "value" );
			Field field = new Field( tag, value );
			vecDef.add( field );
			if ( isDebug()) {
				CmnLog.debug( this, "default " + field );
			}
		}
	}

	public static FIXParser createParser( IConfiguration config )
		throws Exception
	{
		FIXParser ret = null;
		String scls = config.getAttributes().get( "class", null );
		if ( scls == null ) {
			ret = new FIXParser();
		} else {
			Class cls = Class.forName( scls );
			ret = (FIXParser )cls.newInstance();
		}
		if ( ret != null ) {
			ret.init( config );
		}
		return ret;
	}

	public String getHeader( FIXMessage msg, Tag tag )
		throws Exception
	{
		return msg.getHeader().getValue( tag );
	}
	public String getHeader( FIXMessage msg, String tagname )
		throws Exception
	{
		Tag tag = dict.findTagName( tagname );
		if ( tag == null ) {
			throw( new Exception( "tag not found.[" + tagname + "]" ));
		}
		return msg.getHeader().getValue( tag );
	}

	public void putHeader( FIXMessage msg, Tag tag, String value )
		throws Exception
	{
		msg.getHeader().putValue( tag, value );
	}
	public void putHeader( FIXMessage msg, String tagname, String value )
		throws Exception
	{
		Tag tag = dict.findTagName( tagname );
		if ( tag == null ) {
			throw( new Exception( "tag not found.[" + tagname + "]" ));
		}
		msg.getHeader().putValue( tag, value );
	}

	public String getValue( FIXMessage msg, Tag tag )
		throws Exception
	{
		return msg.getValue( tag );
	}
	public String getValue( FIXMessage msg, String tagname )
		throws Exception
	{
		Tag tag = dict.findTagName( tagname );
		if ( tag == null ) {
			throw( new Exception( "tag not found.[" + tagname + "]" ));
		}
		return msg.getValue( tag );
	}

	public void putValue( FIXMessage msg, Tag tag, String value )
		throws Exception
	{
		msg.putValue( tag, value );
	}
	public void putValue( FIXMessage msg, String tagname, String value )
		throws Exception
	{
		Tag tag = dict.findTagName( tagname );
		if ( tag == null ) {
			throw( new Exception( "tag not found.[" + tagname + "]" ));
		}
		msg.putValue( tag, value );
	}

	public String getTrailer( FIXMessage msg, Tag tag )
		throws Exception
	{
		return msg.getTrailer().getValue( tag );
	}
	public String getTrailer( FIXMessage msg, String tagname )
		throws Exception
	{
		Tag tag = dict.findTagName( tagname );
		if ( tag == null ) {
			throw( new Exception( "tag not found.[" + tagname + "]" ));
		}
		return msg.getTrailer().getValue( tag );
	}

	public void putTrailer( FIXMessage msg, Tag tag, String value )
		throws Exception
	{
		msg.getTrailer().putValue( tag, value );
	}
	public void putTrailer( FIXMessage msg, String tagname, String value )
		throws Exception
	{
		Tag tag = dict.findTagName( tagname );
		if ( tag == null ) {
			throw( new Exception( "tag not found.[" + tagname + "]" ));
		}
		msg.getTrailer().putValue( tag, value );
	}

	public FIXMessage parse( byte[] buf, int start, int length )
		throws FIXException
	{
		if ( isDebug()) {
			CmnLog.debug( this, "parse " + start + " " + length );
		}
		FIXMessage ret = new FIXMessage();
		int fldStart = start;
		for ( int pos = start; pos < ( start + length ); pos++ ) {
			if ( buf[pos] != FIXMessage.SOH ) {
				continue;
			}
			if ( pos <= fldStart ) {
				throw( new FIXParseException( "Null Field found.",
						SessionRejectReason.OTHER ));
			}
			Field field;
			field = parseField( ret, buf, fldStart, pos - fldStart );
			if ( field.getTag().getTagId() == Tag.CheckSum.getTagId() ) {
				ret.setLength( pos - start + 1 );
				return ret;
			}
			fldStart = pos + 1;
		}
		return null;
	}

	Field parseField( FIXMessage msg, byte[] buf, int start, int length )
		throws FIXException
	{
		String field = new String( buf, start, length );
		int eqIdx = field.indexOf( "=" );
		if ( eqIdx < 0 ) {
			throw( new FIXParseException( "'=' not found.",
					SessionRejectReason.OTHER ));
		}
		String tagStr = field.substring( 0, eqIdx );
		String value = field.substring( eqIdx + 1 );
		Tag tag = dict.findTag( tagStr );
		if ( isDebug()) {
			CmnLog.debug( this, tag.getName() + "(" + tagStr + ")="
									+ "[" + value + "]" );
		}
		if ( tag.getTagId() == Tag.MsgType.getTagId() ) {
			msg.setMsgType( value );
			msg.setMsgTypeName( dict.getMsgTypeName( value ));
		}
		if ( tag.getTagId() == Tag.BodyLength.getTagId() ) {
			msg.setBodyLength( value );
		}
		if ( tag.getTagId() == Tag.CheckSum.getTagId() ) {
			msg.setCheckSum( value );
		}

		switch ( tag.getPosition() ) {
		case HEADER:
			msg.getHeader().putValue( tag, value );
			break;
		case BODY:
			msg.putValue( tag, value );
			break;
		case TRAILER:
			msg.getTrailer().putValue( tag, value );
			break;
		}
		Field ret = new Field( tag, value );
		return ret;
	}

/*
@@@ CheckSumのチェックはSesionレベルで行うべき
			if ( value.length() != 3 ) {
				throw( new FIXParseException( "invalid CheckSum"
						+ " [" + field + "]",
						SessionRejectReason.VALUE_IS_INCORRECT ));
			}
			try {
				Integer sum = Integer.valueOf( value );
				msg.setCheckSum( value );
			} catch ( Exception ex ) {
				throw( new FIXParseException( "invalid CheckSum format"
						+ " [" + field + "]", ex,
						SessionRejectReason.INCORRECT_DATA_FORMAT ));
			}
*/
/*
	public FIXMessage parse( byte[] buf, int start, int length )
		throws Exception
	{
		boolean	foundTag = false;
		Tag	tag = null;
		String tagstr;
		String value;
		FIXMessage ret = new FIXMessage();

		int tagStart = start;
		int valStart = -1;
		for ( int pos = start; pos < ( start + length ); pos++ ) {
			switch ( buf[pos] ) {
			case '=':
				if ( foundTag ) {
					continue;
				}
				foundTag = true;
				tagstr = new String( buf, tagStart, pos - tagStart );
				tag = dict.findTag( tagstr );
				if ( tag.getTagId() == Tag.CheckSum.getTagId()) {
					if (( pos + 3 + 1 ) > length ) {
						return null;
					}
					value = new String( buf, pos + 1, 3 );
					ret.setCheckSum( value );
					ret.setLength( pos + 1 + 3 );
					if ( isDebug()) {
						CmnLog.debug( this, "CheckSum=" + value
							+ " length=" + ret.getLength() );
					}
					return ret;
				}
				valStart = pos + 1;
				break;
			case FIXMessage.SOH:
				if ( !foundTag ) {
					throw( new FIXParseException( "'=' not found.",
						SessionRejectReason.OTHER ));
				}
				value = new String( buf, valStart, pos - valStart );
				if ( isDebug()) {
					CmnLog.debug( this, tag.getName()
						+ "(" + tag.getTagId() + ")=[" + value + "]" );
				}
				ret.putValue( tag, value );
				foundTag = false;
				tagStart = pos + 1;
				valStart = -1;
			}
		}
		return null;
	}
*/

	public void validate( FIXMessage msg ) throws FIXParseException {
	}

	public byte[] getBytes( FIXMessage msg ) throws Exception {
		putDefault( msg );

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		msg.getBytes( bos, dict );
		byte[] ret = bos.toByteArray();
		return ret;
	}

	void putDefault( FIXMessage msg ) {
		for ( int i = 0; i < vecDef.size(); i++ ) {
			Field field = vecDef.get( i );
			Tag tag = field.getTag();
			switch ( tag.getPosition() ) {
			case HEADER:
				msg.getHeader().putValue( field );
				break;
			case BODY:
				msg.putValue( field );
				break;
			case TRAILER:
				msg.getTrailer().putValue( field );
				break;
			}
		}
	}

	boolean	debug;	// debugフラグ

	/**
	* debugフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDebug( boolean var ) {
		this.debug = var;
	}
	/**
	* debugフラグを取得します。
	* @return	debugフラグ
	*/
	public boolean isDebug() {
		return debug;
	}


}
