package	jp.co.sds_corp.fixconnect.message;

import java.util.Iterator;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;

public class XMLDictionaryLoader extends DictionaryLoader
{
	protected XMLDictionaryLoader() {
	}

	public void init( IConfiguration config ) throws Exception {
		String file = config.getAttributes().get( "file" );
		readDictionary( file );
	}

	protected void readDictionary( String file ) throws Exception {
		DOMConfiguration config = new DOMConfiguration( file );
		TagDictionary dict;
		IConfiguration c;

		c = config.getChild( "FIXHeader" );
		dict = readMessage( c, Tag.Position.HEADER );
		getHeaderDictionary().addAll( dict );

		c = config.getChild( "FIXTrailer" );
		dict = readMessage( c, Tag.Position.TRAILER );
		getTrailerDictionary().addAll( dict );

		Iterator<IConfiguration> itr = config.getChilds( "message" );
		while ( itr.hasNext()) {
			c = itr.next();
			IConfiguration attr = c.getAttributes();
			String msgType = attr.get( "MsgType" );
			String name = attr.get( "name" );
			dict = readMessage( c, Tag.Position.BODY );
			putMessageDictionary( msgType, name, dict );
		}
	}

	protected TagDictionary readMessage( IConfiguration config, Tag.Position pos )
		throws Exception
	{
		TagDictionary ret = new TagDictionary();
		IConfiguration attr = config.getAttributes();
		String name = attr.get( "name", null );
		String msgType = attr.get( "MsgType", null );

		Iterator<IConfiguration> itr;
		itr = config.getChilds( "tag" );
		while ( itr.hasNext()) {
			IConfiguration c = itr.next();
			Tag tag = readTag( c, pos, ret );
			ret.addTag( tag );
			addTag( tag );
		}
		itr = config.getChilds( "component" );
		while ( itr.hasNext()) {
			IConfiguration c = itr.next();
			readComponent( c, pos, ret );
		}
		return ret;
	}

	protected Tag readTag( IConfiguration config, Tag.Position pos, TagDictionary dict )
		throws Exception
	{
		String var;
		Tag tag = new Tag();
		var = config.getAttributes().get( "id" );
		tag.setTagId( Integer.valueOf( var ));
		tag.setName( config.getAttributes().get( "name" ));
		tag.setPosition( pos );
		var = config.getAttributes().get( "required", "false" );
		if ( var.equals( "true" )) {
			tag.setRequired( true );
		}
		return tag;
	}

	protected void readComponent( IConfiguration config, Tag.Position pos,
			TagDictionary dict )
		throws Exception
	{
		IConfiguration attr = config.getAttributes();
		String name = attr.get( "name" );
		IConfiguration c;
		c = config.getChild( "tag" );
		Tag tag = readTag( c, pos, dict );
		dict.addTag( tag );
		addTag( tag );

		IConfiguration crep = config.getChild( "repeat" );
		Iterator<IConfiguration> itr = crep.getChilds( "tag" );
		while ( itr.hasNext()) {
			c = itr.next();
			tag = readTag( c, pos, dict );
			dict.addTag( tag );
			addTag( tag );
		}
	}

/*
	protected void readDictionary( String file ) throws Exception {
		DOMConfiguration dict = new DOMConfiguration( file );
		Iterator<IConfiguration> itr = dict.getChilds( "tag" );
		while ( itr.hasNext()) {
			IConfiguration c = itr.next();
			String var;
			Tag tag = new Tag();
			var = c.getAttributes().get( "id" );
			tag.setTagId( Integer.valueOf( var ));
			tag.setName( c.getAttributes().get( "name" ));
			var = c.getAttributes().get( "type", "" );
			if ( var.equals( "HEAD" )) {
				tag.setPosition( Tag.Position.HEADER );
			} else
			if ( var.equals( "BODY" )) {
				tag.setPosition( Tag.Position.BODY );
			} else
			if ( var.equals( "TRAILER" )) {
				tag.setPosition( Tag.Position.TRAILER );
			}
			var = c.getAttributes().get( "required", "false" );
			if ( var.equals( "true" )) {
				tag.setRequired( true );
			}
			addTag( tag );
		}
	}
*/

}
