package	jp.co.sds_corp.fixconnect.transport;

import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Vector;

import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;

public class Acceptor extends AbstractTransporter
{
	private static final long serialVersionUID = 1L;
	IConfiguration	config;
	ServerSocket	svrSock;
	Vector<FIXSession> vecSession = new Vector<FIXSession>();

	public Acceptor() {
		backlog = 5;
	}

	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		this.config = config;
		IConfiguration attr = config.getAttributes();
		host = attr.get( "host" );
		String var;
		var = attr.get( "port" );
		port = Integer.valueOf( var ).intValue();
		var = attr.get( "backlog", null );
		if ( var != null ) {
			backlog = Integer.valueOf( var ).intValue();
		}
	}

	public void start() {
		try {
			if ( getHost() != null ) {
				CmnLog.info( this, "bind host=[" + host + "]"
					+ " port=[" + port + "]" );
				InetAddress addr = InetAddress.getByName( host );
				svrSock = new ServerSocket( port, backlog, addr );
			} else {
				CmnLog.info( this, "bind port=[" + port + "]" );
				svrSock = new ServerSocket( port, backlog );
			}
			accept();
		} catch ( Exception ex ) {
			CmnLog.error( this, "bind error", ex );
			RuntimeException rex = new RuntimeException( ex );
			throw( rex );
		}
	}

	protected void accept() throws Exception {
		for ( ;; ) {
			Socket sock = svrSock.accept();
			FIXSession session = new FIXSession( sock );
			session.init( config );
			vecSession.add( session );
			session.addMessageReceiver( this );
			session.addEventListener( this );

			CmnLog.info( this, "start session. id=" + session.getSessionId()
					+ " from=" + session.getSocket().getRemoteAddress() );
			session.start();
		}
	}

	public void stop() {
		try {
			svrSock.close();
		} catch ( Exception ex ) {
			CmnLog.error( this, "server socket close error.", ex );
		}
		for ( int i = 0; i < vecSession.size(); i++ ) {
			FIXSession session = vecSession.get( i );
			try {
				session.close();
			} catch ( Exception ex ) {
				CmnLog.error( this, "socket close error.", ex );
			}
		}
	}

	String	host;	// ホスト名
	int	port;	// ポート番号
	int	backlog;	// キュー長

	/**
	* ホスト名を設定します。
	* @param	var		設定するデータ
	*/
	public void setHost( String var ) {
		this.host = var;
	}
	/**
	* ホスト名を取得します。
	* @return	ホスト名
	*/
	public String getHost() {
		return host;
	}

	/**
	* ポート番号を設定します。
	* @param	var		設定するデータ
	*/
	public void setPort( int var ) {
		this.port = var;
	}
	/**
	* ポート番号を取得します。
	* @return	ポート番号
	*/
	public int getPort() {
		return port;
	}

	/**
	* キュー長を設定します。
	* @param	var		設定するデータ
	*/
	public void setBacklog( int var ) {
		this.backlog = var;
	}
	/**
	* キュー長を取得します。
	* @return	キュー長
	*/
	public int getBacklog() {
		return backlog;
	}

}
